#pragma once
#include <SFML/Graphics/Color.hpp>
#include <SFML/Graphics/Font.hpp>
#include <SFML/Graphics/RenderWindow.hpp>
#include <SFML/Graphics/Text.hpp>
#include <SFML/Graphics/Sprite.hpp>
#include <ftxui/component/component.hpp>
#include <ftxui/screen/screen.hpp>
#include <ftxui/dom/canvas.hpp>
#include "ftxui/component/component.hpp"  // for Button, operator|=, Renderer, Vertical, Modal
#include "ftxui/component/screen_interactive.hpp"  // for ScreenInteractive, Component
#include "ftxui/dom/elements.hpp"  // for operator|, separator, text, size, Element, vbox, border, GREATER_THAN, WIDTH, center, HEIGHT
#include <locale>
#include <string>
#include "map.hpp"
#include "dinkyecs.hpp"
#include "components.hpp"
#include "sound.hpp"
#include "render.hpp"
#include "panel.hpp"
#include "lights.hpp"
#include "levelmanager.hpp"

using std::string;
using ftxui::Canvas, ftxui::Component, ftxui::Screen, ftxui::Button;
using lighting::LightRender;

struct ActionLog {
  std::deque<std::string> messages;

  void log(std::string msg) {
    messages.push_front(msg);
    if(messages.size() > 20) {
      messages.pop_back();
    }
  }
};

struct UnDumbTSS {
  sf::Texture texture;
  sf::Sprite sprite;
  sf::Shader shader;

  sf::Shader& load_shader(string filename) {
    bool good = shader.loadFromFile(filename, sf::Shader::Fragment);
    dbc::check(good, "shader could not be loaded");
    return shader;
  }
};

class DeathUI : public Panel {
  public:
  Component $render = nullptr;
  Component $exit_button = nullptr;
  std::string $quip = "You died like a dog.";

  DeathUI() :
    Panel(INVENTORY_PIXEL_X, INVENTORY_PIXEL_Y, INVENTORY_WIDTH, INVENTORY_HEIGHT) {}

  void create_render();
};

class NextLevelUI : public Panel {
  public:
  Component $render = nullptr;
  Component $yes_button = nullptr;
  Component $no_button = nullptr;
  std::string $message = "Are you ready to go further down?";

  NextLevelUI() : Panel(0, 0, NEXT_LEVEL_WIDTH, NEXT_LEVEL_HEIGHT) {}

  void create_render();
};

class InventoryUI : public Panel {
  public:

  int $selected = 0;
  Component $inventory_box;
  Component $inventory_render;
  Component $inventory_table;
  GameLevel $level;
  std::vector<std::string> $menu_list;
  std::string $item_text = "No item selected.";

  InventoryUI(GameLevel level) :
    Panel(INVENTORY_PIXEL_X, INVENTORY_PIXEL_Y, INVENTORY_WIDTH, INVENTORY_HEIGHT),
    $level(level)
  {}

  void create_render();

  void update_level(GameLevel &level) { $level = level; }

  void update_menu_list(components::Inventory& inventory);
};

class StatusUI : public Panel {
  public:
  ActionLog $log;
  string $status_text = "NOT DEAD";
  GameLevel $level;

  StatusUI(GameLevel level) :
    Panel(0, 0, STATUS_UI_WIDTH, STATUS_UI_HEIGHT),
    $log({{"Welcome to the game!"}}),
    $level(level) {}

  void create_render();
  void log(string msg) {
    $log.log(msg);
  }
  void update_level(GameLevel &level) { $level = level; }
};

class MapViewUI : public Panel {
  public:
    Canvas $canvas;
    GameLevel $level;

    MapViewUI(GameLevel &level);
    void create_render();
    void resize_canvas();
    void draw_map();
    void update_level(GameLevel &level);
};

class GUI {
  LevelManager $level_manager;
  GameLevel $level;

  StatusUI $status_ui;
  MapViewUI $map_view;
  InventoryUI $inventory_ui;
  DeathUI $death_ui;
  NextLevelUI $next_level_ui;

  Canvas $canvas;
  bool $inventory_open = false;
  bool $player_died = false;
  bool $modal_shown = false;
  bool $next_level = false;
  Component $test_button;
  SoundManager $sounds;
  SFMLRender $renderer;
  UnDumbTSS $paused;
  std::vector<Panel*> $active_panels;

public:
  GUI();

  // disable copying
  GUI(GUI &gui) = delete;

  void resize_map(int new_size);
  void create_renderer();
  void render_scene();
  bool modal_ui_events();
  bool game_ui_events();
  bool handle_ui_events();
  void handle_world_events();
  void draw_screen(bool clear=true, float map_off_x=0.0f, float map_off_y=0.0f);
  void run_systems();
  void save_world();
  void shake();
  void shutdown();
  void next_level();
  int main(bool run_once=false);

  void pause_screen();
  void draw_paused();
  void init_shaders();
  void toggle_modal(Panel *panel, bool &is_open_out);
};
